@@ -55,6 +55,7 @@
 
 #include "compat.h"
 #include "fmfconv.h"
+#include "fmfconv_compr.h"
 #include "movie_tables.h"
 
 #define PROGRAM_NAME "fmfconv"
@@ -64,15 +65,6 @@
 
 const char *progname;
 
-#ifdef USE_ZLIB
-#define ZBUF_SIZE 1024
-#define ZBUF_INP_SIZE 512
-z_stream zstream;			/* zlib struct */
-unsigned char zbuf_o[ZBUF_SIZE];	/* zlib output buffer */
-unsigned char zbuf_i[ZBUF_INP_SIZE];	/* zlib input buffer */
-int fmf_compr_feof = 0;
-#endif	/* USE_ZLIB */
-
 #ifdef USE_LIBJPEG
 extern int jpg_dctfloat;
 extern int jpg_idctfast;
@@ -114,7 +106,8 @@ int fmfconv_stop = 0;
 FILE *inp_file = NULL, *out = NULL, *snd = NULL;
 int out_to_stdout = 0;
 
-type_t inp_t = TYPE_UNSET, out_t = TYPE_UNSET, snd_t = TYPE_UNSET, scr_t = TYPE_UNSET, prg_t = TYPE_NONE;
+type_t inp_t = TYPE_UNSET, out_t = TYPE_UNSET, snd_t = TYPE_UNSET, prg_t = TYPE_NONE;
+fmf_screen_type scr_t;
 
 const char *inp_name = NULL;
 const char *out_name = NULL;
@@ -196,7 +189,8 @@ libspectrum_qword time_sec = 0, time_frm = 0;	/* current time (input) no */
 libspectrum_qword drop_no = 0;	/* dropped frame no */
 libspectrum_qword dupl_no = 0;	/* duplicated frame no */
 libspectrum_qword output_no = 0;	/* output frame no */
-int frm_scr = 0, frm_rte = 0, frm_mch = 0; /* screen type, frame rate, machine type (A/B/C/D) */
+int frm_scr = 0, frm_rte = 0; /* screen type, frame rate */
+fmf_machine_type frm_mch; /* machine type (A/B/C/D) */
 int frm_slice_x, frm_slice_y, frm_slice_w, frm_slice_h;
 int frm_w = -1, frm_h = -1;
 
@@ -207,7 +201,6 @@ int out_header_ok = 0;			/* output header ok? */
 int out_chn = 2, out_rte = -1, out_fsz, out_len;	/* by default convert sound to 2 channel (STEREO) */
 
 /* fmf variables */
-int fmf_compr = 0;			/* fmf compressed or not */
 int fmf_little_endian = 0;		/* little endian PCM */
 int snd_little_endian = -1;			/* little endian PCM */
 libspectrum_qword fmf_slice_no = 0;
@@ -215,7 +208,7 @@ libspectrum_qword fmf_sound_no = 0;
 
 /* sound variables */
 #define SOUND_BUFF_MAX_SIZE 65536	/* soft max size of collected sound samples */
-type_t snd_enc;				/* sound type (pcm/alaw) */
+fmf_sound_type snd_enc;			/* sound type (pcm/alaw) */
 int snd_rte, snd_chn, snd_fsz, snd_len;	/* sound rate (Hz), sound channels (1/2), sound length in byte  */
 int snd_frg = 0;			/* sound fragment len from previous resample in bytes */
 int snd_header_ok = 0;			/* sound header ok? */
@@ -294,77 +287,6 @@ fread_buff( libspectrum_byte *ptr, size_t size, int how )	/* read from inp, how
   return 1;
 }
 
-#ifdef USE_ZLIB
-static int
-feof_compr( FILE *f )
-{
-  if( fmf_compr ) {
-    if( fmf_compr_feof )
-      return 1;
-    if( zstream.avail_in > 0 || zstream.avail_out != ZBUF_SIZE )
-      return 0;
-    fmf_compr_feof = feof( f );
-    return fmf_compr_feof;
-  } else {
-    return feof( f );
-  }
-}
-
-int fget_read = 0;
-
-static int
-fgetc_compr( FILE *f )
-{
-  int s;
-
-  if( fmf_compr ) {
-    if( zstream.avail_out == ZBUF_SIZE ) {
-      fget_read++;
-      zstream.next_out = zbuf_o;
-      do {
-	if( zstream.avail_in == 0 && !fmf_compr_feof ) {
-	  zstream.avail_in = fread( zbuf_i, 1, ZBUF_INP_SIZE, inp_file );
-	  zstream.next_in = zbuf_i;
-	  if( zstream.avail_in == 0 )
-	    fmf_compr_feof = 1;
-	}
-	s = inflate( &zstream, fmf_compr_feof ? Z_FINISH : Z_SYNC_FLUSH );
-      } while ( zstream.avail_out != 0 && s == Z_OK );
-      zstream.next_out = zbuf_o;
-    }
-    if( zstream.avail_out == ZBUF_SIZE )	/* end of file */
-      return -1;
-    zstream.avail_out++;
-    return *( zstream.next_out++ );    /* !!! we use it for own purpose */
-  } else {
-    return fgetc( f );
-  }
-}
-
-static int
-fread_compr( void *buff, size_t n, size_t m, FILE *f )
-{
-  size_t i;
-  int d;
-  char *b = buff;
-
-  if( fmf_compr ) {
-    for( i = n * m; i > 0; i-- ) {
-      if( ( d = fgetc_compr( f ) ) == -1 )	/****FIXME may too slow */
-        return ( n * m - i ) / n;
-      *b++ = d;
-    }
-    return m;
-  } else {
-    return fread( b, n, m, f );
-  }
-}
-#else	/* USE_ZLIB */
-#define fgetc_compr fgetc
-#define fread_compr fread
-#define feof_compr feof
-#endif	/* USE_ZLIB */
-
 libspectrum_dword
 swap_endian_dword( libspectrum_dword d )
 {
@@ -427,7 +349,7 @@ law_2_pcm( void )
 #else
   snd_little_endian = 1;
 #endif
-  snd_enc = TYPE_PCM;
+  snd_enc = PCM;
   snd_fsz <<= 1;		/* 1byte -> 2byte */
   snd_len <<= 1;		/* 1byte -> 2byte */
 
@@ -461,7 +383,7 @@ mono_2_stereo( void )
 
   if( ( err = alloc_sound_buff( snd_len * 2 ) ) ) return err;
 
-  if( snd_enc == TYPE_PCM ) {
+  if( snd_enc == PCM ) {
     libspectrum_signed_word *s;
     libspectrum_signed_word *t;
     for( s = sound16 + snd_len / 2 - 1, t = sound16 + snd_len - 1 ;
@@ -498,7 +420,7 @@ stereo_2_mono( void )
 /* we have to skip resample fragment space */
   sound8 += snd_frg;
   sound16 = (void *)sound8;
-  if( snd_enc == TYPE_PCM ) {
+  if( snd_enc == PCM ) {
     libspectrum_signed_word *s;
     libspectrum_signed_word *t;
     buff_end = sound16 + snd_len / 2;
@@ -561,7 +483,7 @@ resample_sound( void )
     memmove( snd8_r, sound8 + snd_frg, snd_len );
     snd16_r = (void *)snd8_r;
 
-    if( snd_enc == TYPE_PCM )
+    if( snd_enc == PCM )
       snd16_w += snd_cpy * snd_chn;
     else
       snd8_w += snd_cpy * snd_chn;
@@ -574,13 +496,13 @@ resample_sound( void )
     if( snd_cpy ) { /*   [----    ######] [lrlrlr]  */
       int n = snd_cpy + 1;
 
-      if( snd_enc == TYPE_PCM )
+      if( snd_enc == PCM )
         snd16_w -= snd_cpy * snd_chn;
       else
         snd8_w -= snd_cpy * snd_chn;
 
       while( snd_cpy ) {
-        if( snd_enc == TYPE_PCM ) {
+        if( snd_enc == PCM ) {
           *snd16_w = last_l + (*snd16_r - last_l) * (n - snd_cpy) / n;
           snd16_w++;
         } else {
@@ -588,7 +510,7 @@ resample_sound( void )
           snd8_w++;
         }
         if( snd_chn == 2 ) {
-          if( snd_enc == TYPE_PCM ) {
+          if( snd_enc == PCM ) {
             *snd16_w = last_r + (*(snd16_r+1) - last_r) * (n - snd_cpy) / n;
             snd16_w++;
           } else {
@@ -600,7 +522,7 @@ resample_sound( void )
       }
     }
     while( fmf_rte <= 0 ) {
-      if( snd_enc == TYPE_PCM ) {
+      if( snd_enc == PCM ) {
         if( snd16_w != snd16_r )
           *snd16_w = *snd16_r; /* really we do not need to store copies ... */
         snd16_w++;
@@ -610,7 +532,7 @@ resample_sound( void )
         snd8_w++;
       }
       if( snd_chn == 2 ) {
-        if( snd_enc == TYPE_PCM ) {
+        if( snd_enc == PCM ) {
           if( snd16_w != snd16_r + 1 )
             *snd16_w = *(snd16_r + 1);
           snd16_w++;
@@ -623,7 +545,7 @@ resample_sound( void )
       fmf_rte += snd_rte;
       snd_cpy++;
     }
-    if( snd_enc == TYPE_PCM ) {
+    if( snd_enc == PCM ) {
       last_l = last_r = *snd16_r;
       snd16_r++;
     } else {
@@ -631,7 +553,7 @@ resample_sound( void )
       snd8_r++;
     }
     if( snd_chn == 2 ) {
-      if( snd_enc == TYPE_PCM ) {
+      if( snd_enc == PCM ) {
         last_r = *snd16_r;
         snd16_r++;
       } else {
@@ -643,7 +565,7 @@ resample_sound( void )
     if( snd_cpy ) snd_cpy--;
   }
   snd_frg = snd_cpy * snd_fsz;
-  new_len = ( snd_enc == TYPE_PCM ? (void *)snd16_w : (void *)snd8_w ) -
+  new_len = ( snd_enc == PCM ? (void *)snd16_w : (void *)snd8_w ) -
             (void *)sound8;
   new_len -= snd_frg;
   printi( 3, "resample_sound(): %d sample converted to %ld (%dHz -> %dHz)\n",
@@ -1041,21 +963,21 @@ open_inp( void )
 
   if( inp_t == TYPE_UNSET ) {	/* try to identify the file type */
     fread_buff( fhead, 4, INTO_BUFF );		/* check file type */
-    if( memcmp( fhead, "FMF_", 4 ) == 0 )
-      inp_t = TYPE_FMF;
-    else
-      inp_t = TYPE_SCR;
+    if( memcmp( fhead, "FMF_", 4 ) != 0 ) {
+      printe( "Cannot identify input file '%s'...\n", inp_name );
+      return ERR_OPEN_INP;
+    }
+    inp_t = TYPE_FMF;
   }
-  do_now = inp_t == TYPE_FMF ? DO_HEAD : DO_SLICE;
-  printi( 0, "open_inp(): Input file (%s) opened as %s file.\n", inp_name,
-          inp_t == TYPE_FMF ? "FMF" : "SCR" );
+  do_now = DO_HEAD;
+  printi( 0, "open_inp(): Input file (FMF) opened as %s file.\n", inp_name );
   return 0;
 }
 
 static void
-setup_frame_wh( void )
+setup_frame_wh( libspectrum_byte screen_type )
 {
-  if( ( scr_t = fhead[5] ) == TYPE_HRE ) {	/* screen type => $ R C X */
+  if( ( scr_t = get_screen_type( screen_type ) ) == HIRES ) {	/* screen type => $ R C X */
     if( frm_w != 640 ) {
       frm_w = 640; frm_h = 480;
       pix_yuv[1] = &pix_rgb[ 640 * 480 ];
@@ -1101,51 +1023,34 @@ check_fmf_head( void )
     printe( "Wrong Frame rate '%d', sorry...\n", fhead[4] );
     return ERR_CORRUPT_INP;
   }
-  if( fhead[5] != TYPE_ZXS && fhead[5] != TYPE_TXS && fhead[5] != TYPE_HRE &&
-      fhead[5] != TYPE_HCO ) {
-    printe( "Unknown Screen$ type '%d', sorry...\n", fhead[5] );
-    return ERR_CORRUPT_INP;
-  }
-  if( fhead[6] < 'A' || fhead[6] > 'E' ) {
-    printe( "Unknown Machine type '%d', sorry...\n", fhead[6] );
-    return ERR_CORRUPT_INP;
-  }
   frm_rte = fhead[4];				/* frame rate (1:#) */
-  setup_frame_wh();
-  frm_mch = fhead[6];				/* machine type */
+  setup_frame_wh( fhead[5] );
+  frm_mch = get_machine_type( fhead[6] );	/* machine type */
 
-  inp_fps = machine_timing[frm_mch - 'A'] / frm_rte;	/* real input fps * 1000 frame / 1000s */
+  inp_fps = machine_timing[frm_mch] / frm_rte;	/* real input fps * 1000 frame / 1000s */
   if( out_fps <= 0 ) out_fps = inp_fps;	/* later may change */
 
 /* Check initial sound parameters */
-  if( fhead[7] != 'P' && fhead[7] != 'A' ) {
-    printe( "Unknown FMF sound encoding type '%d', sorry...\n", fhead[7] );
-    return ERR_CORRUPT_INP;
-  }
   if( fhead[8] + ( fhead[9] << 8 ) < 1 ) {
     printe( "Wrong FMF sound rate '%d', sorry...\n", fhead[8] + ( fhead[9] << 8 ) );
     return ERR_CORRUPT_INP;
   }
-  if( fhead[10] != 'M' && fhead[10] != 'S' ) {
-    printe( "Unknown FMF sound channels type '%d', sorry...\n", fhead[10] );
-    return ERR_CORRUPT_INP;
-  }
 
-  snd_enc = fhead[7];		/* P U A */
+  snd_enc = get_sound_type( fhead[7] );		/* P U A */
   snd_rte = fhead[8] + ( fhead[9] << 8 );
-  snd_chn = fhead[10] == TYPE_STEREO ? 2 : 1;
-  snd_fsz = ( snd_enc == TYPE_PCM ? 2 : 1 ) * snd_chn;
-  if( snd_enc == TYPE_PCM ) snd_little_endian = fmf_little_endian;
+  snd_chn = get_sound_channels_count( fhead[10] );
+  snd_fsz = ( snd_enc == PCM ? 2 : 1 ) * snd_chn;
+  if( snd_enc == PCM ) snd_little_endian = fmf_little_endian;
 
   if( sound_raw || out_rte == -1 ) out_rte = snd_rte;
   if( sound_raw ) out_chn = snd_chn;
   do_now = DO_SLICE;
   printi( 1, "check_fmf_head(): file:  FMF V1 %s endian %scompressed.\n",
           fmf_little_endian ? "little" : "big", fmf_compr ? "" : "un" );
-  printi( 1, "check_fmf_head(): video: frame rate = 1:%d frame time: %dus %s machine timing.\n", frm_rte, machine_ftime[frm_mch - 'A'],
-          machine_name[frm_mch - 'A'] );
-  printi( 1, "check_fmf_head(): audio: sampling rate %dHz %c encoded %s sound.\n",
-          snd_rte, snd_enc, snd_chn == 2 ? "stereo" : "mono" );
+  printi( 1, "check_fmf_head(): video: frame rate = 1:%d frame time: %dus %s machine timing.\n", frm_rte, machine_ftime[frm_mch],
+          machine_name[frm_mch] );
+  printi( 1, "check_fmf_head(): audio: sampling rate %dHz %s encoded %s sound.\n",
+          snd_rte, get_sound_type_string( snd_enc ), snd_chn == 2 ? "stereo" : "mono" );
   return 0;
 }
 
@@ -1168,33 +1073,29 @@ fmf_read_frame_head( void )
     printe( "Wrong Frame rate '%d', sorry...\n", fhead[0] );
     return ERR_CORRUPT_INP;
   }
-  if( fhead[1] != TYPE_ZXS && fhead[1] != TYPE_TXS && fhead[1] != TYPE_HRE &&
-      fhead[1] != TYPE_HCO ) {
-    printe( "Unknown Screen$ type '%d', sorry...\n", fhead[1] );
-    return ERR_CORRUPT_INP;
-  }
+  get_screen_type( fhead[1] );
   if( fhead[2] < 'A' || fhead[2] > 'E' ) {
     printe( "Unknown Machine type '%d', sorry...\n", fhead[2] );
     return ERR_CORRUPT_INP;
   }
 
-  setup_frame_wh();
+  setup_frame_wh( fhead[1] );
 
-  if( frm_rte != fhead[0] || frm_mch != fhead[2] ) {	/* recalculate timings */
+  if( frm_rte != fhead[0] || frm_mch != get_machine_type( fhead[2] ) ) {	/* recalculate timings */
     frm_rte = fhead[0];				/* frame rate (1:#) */
-    frm_mch = fhead[2];				/* machine type */
-    inp_fps = machine_timing[frm_mch - 'A'] / frm_rte;	/* real input fps * 1000 frame / 1000s */
+    frm_mch = get_machine_type( fhead[2] );	/* machine type */
+    inp_fps = machine_timing[frm_mch] / frm_rte;	/* real input fps * 1000 frame / 1000s */
     if( out_fps <= 0 ) out_fps = inp_fps;	/* later may change */
   }
   do_now = DO_SLICE;
   frame_no++;
-  time_frm += machine_ftime[frm_mch - 'A'] * frm_rte;
+  time_frm += machine_ftime[frm_mch] * frm_rte;
   while( time_frm >= 1000000 ) {
     time_frm -= 1000000;
     time_sec++;
   }
   printi( 2, "fmf_read_frame_head(): rate = 1:%d frame time: %dus %s machine.\n",
-          frm_rte, machine_ftime[frm_mch - 'A'], machine_name[frm_mch - 'A'] );
+          frm_rte, machine_ftime[frm_mch], machine_name[frm_mch] );
   return 0;
 }
 
@@ -1242,19 +1143,19 @@ fmf_read_sound( void )
     return ERR_CORRUPT_INP;
   }
 
-  if( snd_len && ( snd_enc != fhead[0] ||		/* encoding change */
+  if( snd_len && ( snd_enc != get_sound_type( fhead[0] ) ||		/* encoding change */
 		snd_rte != fhead[1] + ( fhead[2] << 8 ) || /* sampling rate change */
-		snd_chn != ( fhead[3] == TYPE_STEREO ? 2 : 1 ) ) ) { /* channels change */
+		snd_chn != get_sound_channels_count( fhead[3] ) ) ) { /* channels change */
     printi( 3, "fmf_read_sound(): sound parameters changed flush sound buffer\n" );
     do_now = DO_SOUND_FLUSH;
     return 0;
   }
 
   fmf_snd_head_read = 0;
-  snd_enc = fhead[0];		/* P U A */
+  snd_enc = get_sound_type( fhead[0] );		/* P U A */
   snd_rte = fhead[1] + ( fhead[2] << 8 );
-  snd_chn = fhead[3] == TYPE_STEREO ? 2 : 1;
-  snd_fsz = ( snd_enc == TYPE_PCM ? 2 : 1 ) * snd_chn;
+  snd_chn = get_sound_channels_count( fhead[3] );
+  snd_fsz = ( snd_enc == PCM ? 2 : 1 ) * snd_chn;
   len = ( fhead[4] + ( fhead[5] << 8 ) + 1 ) * snd_fsz;
 
   if( sound_stereo == -1 ) out_chn = snd_chn;
@@ -1270,7 +1171,7 @@ fmf_read_sound( void )
     do_now = DO_SOUND;
     return 0;
   }
-  printi( 3, "fmf_read_sound(): %s %dHz %c encoded %s %d samples (%d bytes) sound\n", cut_cut ? "skip" : "store", snd_rte, snd_enc,
+  printi( 3, "fmf_read_sound(): %s %dHz %s encoded %s %d samples (%d bytes) sound\n", cut_cut ? "skip" : "store", snd_rte, get_sound_type_string( snd_enc ),
 		 snd_chn == 2 ? "stereo" : "mono", len/snd_fsz, len );
   return 0;
 }
@@ -1282,7 +1183,7 @@ snd_write_sound( void )
 
   if( snd_len <= 0 ) return 0;
 
-  if( snd_enc != TYPE_PCM && sound_pcm && ( err = law_2_pcm() ) ) return err;
+  if( snd_enc != PCM && sound_pcm && ( err = law_2_pcm() ) ) return err;
   if( snd_chn == 2 && sound_stereo == 0 && ( err = stereo_2_mono() ) ) return err;
   if( ( snd_rte != out_rte || snd_frg ) && ( err = resample_sound() ) ) return err;
   if( snd_chn == 1 && sound_stereo == 1 && ( err = mono_2_stereo() ) ) return err;
@@ -1312,14 +1213,14 @@ fmf_gen_sound( void )
   if( snd_t == TYPE_NONE ) return 0;
 
 /* spectrum frame length * frame rate * freq * framesize 20000 * 9 * 64000 * 4 ~ 5*10^10 so we need 64bit! */
-  len = (libspectrum_qword)len_frag + (libspectrum_qword)machine_ftime[frm_mch - 'A'] * frm_rte * snd_rte * snd_fsz;
+  len = (libspectrum_qword)len_frag + (libspectrum_qword)machine_ftime[frm_mch] * frm_rte * snd_rte * snd_fsz;
   len_frag = len - (len / 1000000) * 1000000;
   len = len / 1000000;
 
   if( ( err = alloc_sound_buff( snd_len + len ) ) ) return err;
-  memset((void *)( sound8 + snd_len ), snd_enc == 'P' ? 0 : 0xd5, len);	/* only A-Law!!!*/
+  memset((void *)( sound8 + snd_len ), snd_enc == PCM ? 0 : 0xd5, len);	/* only A-Law!!!*/
   snd_len += len;
-  printi( 3, "fmf_gen_sound(): store %dHz %c encoded %s %d samples (%d bytes) silence\n", snd_rte, snd_enc,
+  printi( 3, "fmf_gen_sound(): store %dHz %s encoded %s %d samples (%d bytes) silence\n", snd_rte, get_sound_type_string( snd_enc ),
 		 snd_chn == 2 ? "stereo" : "mono", (int)len/snd_fsz, (int)len );
   if( snd_len >= SOUND_BUFF_MAX_SIZE ) {
     if( ( err = snd_write_sound() ) ) return err;
@@ -1425,7 +1326,7 @@ fmf_read_screen( void )
   if( ( err = fmf_read_slice_blokk( zxscr + frm_slice_y * SCR_PITCH + frm_slice_x,
 			       frm_slice_w, frm_slice_h ) ) ) return err;	/* we store it with cont. */
 /* read bitmap2 if HighResolution Timex */
-  if( scr_t == TYPE_HRE && ( err = fmf_read_slice_blokk( zxscr + 9600 +
+  if( scr_t == HIRES && ( err = fmf_read_slice_blokk( zxscr + 9600 +
 			     frm_slice_y * SCR_PITCH + frm_slice_x,
 			     frm_slice_w, frm_slice_h ) ) ) return err;	/* we store it with cont. */
 /*read attrib */
@@ -1475,14 +1376,6 @@ we have to handle cut here
   return err;
 }
 
-static int
-scr_read_scr( void )
-{
-  /* FIXME: implement SCR support or remove this. Unknown file format is
-     interpreted as SCR file */
-  return 1;
-}
-
 /* store RGB/YUV/Paletted color/grayscale pixel*/
 static void
 pix_pix( int *idx, int xx, int i )
@@ -1530,7 +1423,7 @@ out_2_pix( void )
 
   w = frm_slice_w;
   for( h = frm_slice_h; h > 0; h--, y++, w = frm_slice_w ) {
-    if( scr_t != TYPE_HRE )
+    if( scr_t != HIRES )
       idx = y * OUT_PITCH + x * 8 ;		/* 8pixel/data */
     else
       idx = y * OUT_PITCH * 2 + x * 16 ;	/* 16 pixel/data*/
@@ -1544,12 +1437,12 @@ out_2_pix( void )
     attr   = &attrs[SCR_PITCH * y + x];
     for( ; w > 0; w--, bitmap++, attr++ ) {
       int px, fx, ix = *attr;
-      int bits = scr_t == TYPE_HRE ? ( *bitmap << 8 ) + *(bitmap + 9600) : *bitmap;
+      int bits = scr_t == HIRES ? ( *bitmap << 8 ) + *(bitmap + 9600) : *bitmap;
 
       fx = ( ix & 0x80 ) * inv;
       px = ( ( ix & 0x38 ) >> 3 ) + ( ( ix & 0x40 ) ? 8 : 0 );
       ix = ( ix & 0x07 ) + ( ( ix & 0x40 ) ? 8 : 0 );
-      for( i = scr_t == TYPE_HRE ? 16 : 8; i > 0; i-- ) {
+      for( i = scr_t == HIRES ? 16 : 8; i > 0; i-- ) {
         pix_pix( &idx, ( bits ^ fx ) & 128 ? ix : px, i );
         bits <<= 1;
       }
@@ -2183,13 +2076,7 @@ main( int argc, char *argv[] )
   else if( sound_stereo == 0 ) out_chn = 1;
 
 #ifdef USE_ZLIB
-  zstream.zalloc = Z_NULL;
-  zstream.zfree = Z_NULL;
-  zstream.opaque = Z_NULL;
-  zstream.avail_in = 0;
-  zstream.next_in = Z_NULL;
-  inflateInit( &zstream );
-  zstream.avail_out = ZBUF_SIZE;
+  fmf_compr_init();
 #endif	/* USE_ZLIB */
 
 #ifdef HAVE_SIGNAL
@@ -2213,11 +2100,7 @@ main( int argc, char *argv[] )
       if( ( err = fmf_read_frame_head() ) ) eop = 1;
       break;
     case DO_SLICE:					/* read next fmf slice or 'N' or 'S' or 'X' */
-      if( inp_t == TYPE_FMF ) {
-        if( ( err = fmf_read_slice() ) ) eop = 1;
-      } else {						/* read SCR file */
-        if( ( err = scr_read_scr() ) ) eop = 1;
-      }
+      if( ( err = fmf_read_slice() ) ) eop = 1;
       if( out_t != TYPE_NONE ) {		/* convert slice to RGB or YUV if needed */
         if( out_t >= TYPE_PPM ) out_2_pix();
       }
@@ -2266,7 +2149,7 @@ main( int argc, char *argv[] )
   }
 
 #ifdef USE_ZLIB
-  inflateEnd( &zstream );
+  fmf_compr_end();
 #endif	/* USE_ZLIB */
 
   if( ( out_t >= TYPE_SCR && out_t <= TYPE_JPEG ) && out_name ) unlink( out_name );
