@@ -24,6 +24,10 @@
 #ifndef FMFCONV_H
 #define FMFCONV_H
 
+#include <libspectrum.h>
+
+#include "fmfconv_types.h"
+
 #define FMFCONV_VER_MAJOR 0
 #define FMFCONV_VER_MINOR 5
 #define FMFCONV_RC_VERSION FMFCONV_VER_MAJOR,FMFCONV_VER_MINOR,0,0
@@ -110,17 +114,6 @@ typedef enum {
   TYPE_CUTFROM,
   TYPE_CUT,
 
-  TYPE_ZXS = '$',	/* screen type $ -> normal Spectrum */
-  TYPE_TXS = 'X',	/* X -> Timex normal */
-  TYPE_HRE = 'R',	/* R -> Timex HighRes */
-  TYPE_HCO = 'C',	/* C -> Timex HiColor */
-
-  TYPE_PCM = 'P',	/* 16 bit signed PCM */
-  TYPE_ALW = 'A',	/* 8 bit signed A-Law */
-  TYPE_ULW = 'U',	/* 8 bit signed u-Law */
-
-  TYPE_MONO= 'M',
-  TYPE_STEREO='S',
 
 
 } type_t;
@@ -132,10 +125,12 @@ extern int out_to_stdout;
 
 extern int frm_slice_x, frm_slice_y, frm_slice_w, frm_slice_h;
 extern int frm_w, frm_h;
-extern int frm_fps, frm_mch;
+extern int frm_fps;
+extern fmf_machine_type frm_mch;
 extern libspectrum_qword output_no;	/* output frame no */
 
-extern type_t scr_t, yuv_t, out_t, snd_t;
+extern type_t yuv_t, out_t, snd_t;
+extern fmf_screen_type scr_t;
 
 extern const char *out_name;
 extern int out_w, out_h;
@@ -147,7 +142,7 @@ extern int out_chn, out_rte, out_fsz, out_len;
 extern libspectrum_signed_byte *sound8;	/* sound buffer for x-law */
 extern libspectrum_signed_word *sound16;	/* sound buffer for pcm */
 
-extern type_t snd_enc;				/* sound type (pcm/alaw/ulaw) */
+extern fmf_sound_type snd_enc;			/* sound type (pcm/alaw/ulaw) */
 extern int snd_rte, snd_chn, snd_fsz, snd_len;	/* sound rate (Hz), sound channels (1/2), sound length in byte  */
 extern int snd_header_ok;			/* sound header ok? */
 extern int snd_little_endian;
