@@ -39,6 +39,7 @@
 
 #include <libspectrum.h>
 
+#include "compat.h"
 #include "utils.h"
 
 #define PROGRAM_NAME "rzxdump"
@@ -48,6 +49,7 @@ char *progname;
 const char *rzx_signature = "RZX!";
 
 size_t block_number;
+static int dump_snapshots = 0;
 
 libspectrum_word read_word( unsigned char **ptr );
 libspectrum_dword read_dword( unsigned char **ptr );
@@ -58,7 +60,8 @@ do_file( const char *filename );
 static int
 read_creator_block( unsigned char **ptr, unsigned char *end );
 static int
-read_snapshot_block( unsigned char **ptr, unsigned char *end );
+read_snapshot_block( unsigned char **ptr, unsigned char *end,
+                     const char *rzx_filename );
 static int
 read_input_block( unsigned char **ptr, unsigned char *end );
 static int read_sign_start_block( unsigned char **ptr, unsigned char *end );
@@ -103,6 +106,7 @@ show_help( void )
     "List contents of Sinclair ZX Spectrum input recording files.\n"
     "\n"
     "Options:\n"
+    "  -s             Dump snapshots to files.\n"
     "  -h, --help     Display this help and exit.\n"
     "  -V, --version  Output version information and exit.\n"
     "\n"
@@ -127,10 +131,14 @@ int main( int argc, char **argv )
 
   progname = argv[0];
 
-  while( ( c = getopt_long( argc, argv, "hV", long_options, NULL ) ) != -1 ) {
+  while( ( c = getopt_long( argc, argv, "shV", long_options, NULL ) ) != -1 ) {
 
     switch( c ) {
 
+    case 's':
+      dump_snapshots = 1;
+      break;
+
     case 'h':
       show_help();
       return 0;
@@ -160,7 +168,8 @@ int main( int argc, char **argv )
   }
 
   if( argc < 1 ) {
-    fprintf( stderr, "%s: Usage: %s <rzx file(s)>\n", progname, progname );
+    fprintf( stderr, "%s: Usage: %s [OPTION] <rzx file>...\n", progname,
+             progname );
     fprintf( stderr, "Try `%s --help' for more information.\n", progname );
     return 1;
   }
@@ -221,7 +230,7 @@ do_file( const char *filename )
     case 0x10: error = read_creator_block( &ptr, end ); break; 
     case 0x20: error = read_sign_start_block( &ptr, end ); break;
     case 0x21: error = read_sign_end_block( &ptr, end ); break;
-    case 0x30: error = read_snapshot_block( &ptr, end ); break;
+    case 0x30: error = read_snapshot_block( &ptr, end, filename ); break;
     case 0x80: error = read_input_block( &ptr, end ); break;
 
     default:
@@ -309,11 +318,73 @@ print_ascii_string( unsigned char *data, size_t length )
   }
 }
 
+static char *
+get_snap_filename( const char *rzx_archive, const char *extension,
+                   size_t number )
+{
+  char *buffer, *buffer2, *last_dot, *snap_filename;
+  size_t snap_filename_len;
+
+  /* Get the basename without the last extension */
+
+  buffer = strdup( rzx_archive );
+  if( !buffer ) {
+    fprintf( stderr, "out of memory at %s:%d", __FILE__, __LINE__ );
+    return NULL;
+  }
+
+  buffer2 = compat_basename( buffer );
+  last_dot = strrchr( buffer2, '.' ); if( last_dot ) *last_dot = '\0';
+
+  snap_filename_len = strlen( buffer2 ) + 16;
+  snap_filename = libspectrum_new( char, snap_filename_len );
+
+  if( extension )
+    snprintf( snap_filename, snap_filename_len, "%s_%lu.%s", buffer2, number,
+              extension );
+  else
+    snprintf( snap_filename, snap_filename_len, "%s_%lu", buffer2, number );
+
+  free( buffer );
+
+  return snap_filename;
+}
+
+static void
+write_snapshot( const char *filename, const unsigned char *buffer,
+                size_t length, int compressed )
+{
+  libspectrum_error error;
+
+  if( !compressed ) {
+    write_file( filename, buffer, length );
+  } else {
+
+#ifdef LIBSPECTRUM_SUPPORTS_ZLIB_COMPRESSION
+    size_t new_length;
+    libspectrum_byte *new_buffer = NULL;
+
+    error = libspectrum_zlib_inflate( buffer, length, &new_buffer,
+                                      &new_length );
+    if( !error ) {
+      write_file( filename, new_buffer, new_length );
+      libspectrum_free( new_buffer );
+    }
+#else
+    fprintf( stderr, "%s: zlib not available to decompress snapshot %s\n",
+             progname, filename );
+#endif
+
+  }
+}
+
 static int
-read_snapshot_block( unsigned char **ptr, unsigned char *end )
+read_snapshot_block( unsigned char **ptr, unsigned char *end,
+                     const char *rzx_filename )
 {
   size_t block_length, path_length;
   libspectrum_dword flags;
+  char *snap_filename, *snap_extension;
 
   if( end - *ptr < 16 ) {
     fprintf( stderr,
@@ -330,11 +401,15 @@ read_snapshot_block( unsigned char **ptr, unsigned char *end )
 
   flags = read_dword( ptr );
   printf( "  Flags: %lu\n", (unsigned long)flags );
-  printf( "  Snapshot extension: `%s'\n", *ptr ); (*ptr) += 4;
+
+  snap_extension = (char *) *ptr;
+  printf( "  Snapshot extension: `%s'\n", snap_extension );
+  (*ptr) += 4;
+
   printf( "  Snap length: %d bytes\n", read_dword( ptr ) );
 
   /* Snapshot descriptor */
-  if( ( flags & 0x01 ) && !( flags & 0x02 ) ) {
+  if( ( flags & 0x03 ) == 1 ) {
     if( end - *ptr < 5 ) {
       fprintf( stderr, "%s: Not enough bytes for snapshot block\n", progname );
       return 1;
@@ -352,6 +427,13 @@ read_snapshot_block( unsigned char **ptr, unsigned char *end )
   }
 
   /* Snapshot data */
+  if( dump_snapshots ) {
+    snap_filename = get_snap_filename( rzx_filename, snap_extension,
+                                       block_number - 1 );
+    write_snapshot( snap_filename, *ptr, block_length - 17, flags & 0x02 );
+    free( snap_filename );
+  }
+
   (*ptr) += block_length - 17;
 
   return 0;
